// On-line estimation of regression model
// initial statistics are set from ni measurements
// - try to change ni and compare the beginning of est. parameters 
// - try to vary model parameters and model noise variance
exec("ScIntro.sce",-1), mode(0)

ni=5; nd=200;                      // # of initial and measured data
b0=1; a1=.6; b1=-.2; k=-1.5; r=2;  // coefficients of simulation
y=zeros(1,ni+nd);
u=1*rand(1,ni+nd,'n');             // u=0 est. fails

// SIMULATION
for t=2:ni+nd
  e=sqrt(r)*rand(1,1,'norm');
  y(t)=b0*u(t)+a1*y(t-1)+b1*u(t-1)+k+e;   // simulated output
end

subplot(121)
plot(1:(ni+nd),y,'.:',1:(ni+nd),u,'.:')
set(gcf(),'position',[100 50 900 500]);

// INITIAL ESTIMATION from measured data
V=1e-8*eye(5,5); Ett=[];
for t=2:ni
  Ps=[y(t) u(t) y(t-1) u(t-1) 1];
  V=V+Ps'*Ps;
end

// REGULAR ESTIMATION from measured data
for t=(2:nd)+ni
  Ps=[y(t) u(t) y(t-1) u(t-1) 1];
  V=V+Ps'*Ps;       // update of statistics
  Vy=V(1,1);        // decomposition of inf. matrix
  Vyp=V(2:$,1);
  Vp=V(2:$,2:$);
  Eth=inv(Vp+1e-8*eye(Vp))*Vyp;    // point estimates
  Ett=[Ett Eth];                   // remembering (for graph)
end

subplot(122)
plot(Ett') 
plot([nd-50 nd],[b0 b0],'--',[nd-50 nd],[a1 a1],'--',...
  [nd-50 nd],[b1 b1],'--',[nd-50 nd],[k k],'--')
legend('b0','a1','b1','k');
