// Prediction with discrete model (off-line)
// Vary: - length of prediction
//       - uncertainty of the simulated model
exec("ScIntro.sce",-1),mode(0)

nd=100;
np=5;
th1=[0.98 0.01 0.04 0.97]';        // parameters for simulation (for y=1)
th=[th1 1-th1];                    // all parameters
u=(rand(1,nd)>.3)+1;               // input
y=ones(1,nd);

// simulation
for t=2:nd
  i=2*(u(t)-1)+y(t-1);             // row of the table
  y(t)=(rand(1,1,'unif')>th(i,1))+1; // output generation
end

// prediction
yj=ones(1,nd);                     // fictitious predicted output
for t=2:(nd-np)
  yj(t)=y(t);                      // at time t start with real output
  for j=1:np
    tj=t+j;                        // future times
    i=2*(u(tj)-1)+yj(tj-1);        // row of the table
    yj(tj)=(rand(1,1,'unif')>th(i,1))+1; // prediction generation
  end
  yp(tj)=yj(tj);                   // np-step predction
end

// Results
disp(' Model parameters')
th

s=(np+3):nd;
plot(s,y(s),'.:',s,yp(s),'rx')
set(gcf(),'position',[400 100 800 400])
set(gca(),"data_bounds",[0 nd+1 .9 2.1])
legend('output','prediction');
title(string(np)+'-steps ahead prediction')
