// Control with scalar 1st order regression model
// - simulated data  y(t)=a*y(t-1)+b*u(t)+k+e(t);
// - state realization of the model for synthesis
// - control on a receding control interval with the length nc
// - following a settpoint s(t)
// Experiments
// - change setpoint and system parameters (slow - quick system)
// - penalization of input variable
// - initial condition for estimation (better or worse initial param.)
// ----------------------------------------------------------
exec SCIHOME/ScIntro.sce, mode(0)

nd=100;                            // number of data to be controlled
nc=5;                              // length of control interval
a=.6; b=1; k=-3;                   // model
aE=.1; bE=.5; kE=1;                //  parameters for simulation
om=1; la=.01;                      // penalizations (output, input)
s=sign(100*sin(18*(1:nd)/nd));     // set-point
Om=diag([1 la 0]);                 // matrix penalization
y(1)=5;                            // initial output
V=1e-8*eye(4,4);                   // initial information matrix

// COMPUTATION OF CONTROL-LAW
S=list();
R=list();
u(1)=0;                            // initial control
for t=2:nd               // loop for comtrol
  N=[bE 1 0]';                     // state-space model
  R(nc+1)=zeros(Om);               // initial condition for dyn.prog.
  for i=nc:-1:1          // loop for receding horizon
    M=[aE 0 kE-s(t); 0 0 0; 0 0 1];// state-space model
    T=R(i+1)+Om;                   // dynamic
    A=N'*T*N;                      //  programming
    B=N'*T*M;
    C=M'*T*M;
    S(i)=inv(A)*B;
    R(i)=C-S(i)'*A*S(i);
  end

// CONTROL REALIZATION
  u(t)=-S(1)*[y(t-1) u(t-1) 1]';             // optimal control value
  y(t)=a*y(t-1)+b*u(t)+k+.1*rand(1,1,'n');   // simulation
  
// ESTIMATION
  Ps=[y(t) y(t-1) u(t) 1]';
  V=V+Ps*Ps';
  Vyp=V(2:4,1);
  Vp=V(2:4,2:4);
  th=inv(Vp)*Vyp;                  // point estimates
  aE=th(1);                        // regression
  bE=th(2);                        //  model
  kE=th(3);                        //  parameters
end

// RESULTS
z=1:nd;
plot(z,y(z),'--',z,u(z),z,s(z),':')
legend('y','u','s');
